@@ -5,7 +5,7 @@ use warnings;
 package Dist::Zilla::Plugin::BumpVersionAfterRelease;
 # ABSTRACT: Bump module versions after distribution release
 
-our $VERSION = '0.009';
+our $VERSION = '0.012';
 
 use Moose;
 with(
@@ -14,9 +14,22 @@ with(
       { default_finders => [ ':InstallModules', ':ExecFiles' ], },
 );
 
+use Dist::Zilla::Plugin::BumpVersionAfterRelease::_Util;
 use namespace::autoclean;
 use version ();
 
+#pod =attr allow_decimal_underscore
+#pod
+#pod Allows use of decimal versions with underscores.  Default is false.  (Version
+#pod tuples with underscores are never allowed!)
+#pod
+#pod =cut
+
+has allow_decimal_underscore => (
+    is  => 'ro',
+    isa => 'Bool',
+);
+
 #pod =attr global
 #pod
 #pod If true, all occurrences of the version pattern will be replaced.  Otherwise,
@@ -43,21 +56,40 @@ has munge_makefile_pl => (
 );
 
 has _next_version => (
-    is      => 'ro',
-    isa     => 'Str',
-    lazy    => 1,
-    builder => '_build__next_version',
+    is       => 'ro',
+    isa      => 'Str',
+    lazy     => 1,
+    init_arg => undef,
+    builder  => '_build__next_version',
 );
 
 sub _build__next_version {
     my ($self) = @_;
     require Version::Next;
     my $version = $self->zilla->version;
-    $self->log_fatal("$version is not a valid version string")
-      unless version::is_lax($version);
+
+    $self->log_fatal(
+        "$version is not an allowed version string (maybe you need 'allow_decimal_underscore')"
+      )
+      unless $self->allow_decimal_underscore
+      ? is_loose_version($version)
+      : is_strict_version($version);
+
     return Version::Next::next_version($version);
 }
 
+around dump_config => sub {
+    my ( $orig, $self ) = @_;
+    my $config = $self->$orig;
+
+    $config->{ +__PACKAGE__ } = {
+        finders => [ sort @{ $self->finder } ],
+        ( map { $_ => $self->$_ ? 1 : 0 } qw(global munge_makefile_pl) ),
+    };
+
+    return $config;
+};
+
 sub after_release {
     my ($self) = @_;
     $self->munge_file($_) for @{ $self->found_files };
@@ -81,7 +113,7 @@ sub munge_file {
     }
 
     if ( $self->rewrite_version( $file, $self->_next_version ) ) {
-        $self->log_debug( [ 'bumped $VERSION in %s', $file->name ] );
+        $self->log_debug( [ 'bumped $VERSION in %s', $file->_original_name ] );
     }
     else {
         $self->log( [ q[Skipping: no "our $VERSION = '...'" found in "%s"], $file->name ] );
@@ -89,9 +121,7 @@ sub munge_file {
     return;
 }
 
-my $assign_regex = qr{
-    our \s+ \$VERSION \s* = \s* (['"])$version::LAX\1 \s* ;
-}x;
+my $assign_regex = assign_re();
 
 sub rewrite_version {
     my ( $self, $file, $version ) = @_;
@@ -102,10 +132,12 @@ sub rewrite_version {
     my $iolayer = sprintf( ":raw:encoding(%s)", $file->encoding );
 
     # read source file
-    my $content = Path::Tiny::path( $file->_original_name )->slurp( { binmode => $iolayer } );
+    my $content =
+      Path::Tiny::path( $file->_original_name )->slurp( { binmode => $iolayer } );
 
-    my $comment = $self->zilla->is_trial ? ' # TRIAL' : '';
-    my $code = "our \$VERSION = '$version';$comment";
+    my $code = "our \$VERSION = '$version';";
+    $code .= "\n\$VERSION = eval \$VERSION;"
+      if $version =~ /_/ and scalar( $version =~ /\./g ) <= 1;
 
     if (
         $self->global
@@ -159,7 +191,7 @@ Dist::Zilla::Plugin::BumpVersionAfterRelease - Bump module versions after distri
 
 =head1 VERSION
 
-version 0.009
+version 0.012
 
 =head1 SYNOPSIS
 
@@ -178,7 +210,14 @@ In your F<dist.ini>:
 
 After a release, this module modifies your original source code to replace an
 existing C<our $VERSION = '1.23'> declaration with the next number after the
-released version as determined by L<Version::Next>.  Only the B<first>
+released version as determined by L<Version::Next>.
+
+By default, versions B<must> be "strict" -- decimal or 3+ part tuple with a
+leading "v".  The C<allow_decimal_underscore> option, if enabled, will also
+allow decimals to contain an underscore.  All other version forms are
+not allowed, including: "v1.2", "1.2.3" and "v1.2.3_4".
+
+Only the B<first>
 occurrence is affected (unless you set the L</global> attribute) and it must
 exactly match this regular expression:
 
@@ -260,8 +299,71 @@ how you might do that.
     allow_dirty_match = ^lib/
     commit_msg = Commit Changes and bump $VERSION
 
+=head2 Using underscore in decimal $VERSION
+
+By default, versions must meet the 'strict' criteria from
+L<version|version.pm>, which does not allow the use of underscores.
+
+If the C<allow_decimal_underscore> options is set to true, you may
+use underscores in decimal versions.  In this case, the following line will
+be added after the C<$VERSION> assignment to ensure the underscore is
+removed at runtime:
+
+    $VERSION = eval $VERSION;
+
+Despite their long history on CPAN, the author does not recommend the use
+of decimal underscore versions with Dist::Zilla, as Dist::Zilla supports
+generating tarballs with a "-TRIAL" part of the name as well as putting a
+C<release_status> in META.json – both of which prevent PAUSE from indexing
+a distribution.
+
+Plus, since this plugin also adds the '# TRIAL' comment on the version
+line, it's obvious in the source that the module is a development release.
+With both source and tarball obviously marked "TRIAL", most of the
+historical need for underscore in a version is taken care of.
+
+Using decimal underscores (with the "eval" hack ) introduces a subtle
+difference between what the C<< MM->parse_version >> thinks the version is
+(and what is in META) and what Perl thinks the version is at runtime.
+
+    Foo->VERSION eq MM->parse_version( $INC{"Foo.pm"} )
+
+This would be false for the version "1.002_003" with
+C<$VERSION = eval $VERSION>.  Much of the toolchain has heuristics to
+deal with this, but it may be an issue depending on exactly what
+version of toolchain modules you have installed.  You can avoid all of
+it by just not using underscores.
+
+On the other hand, using underscores and B<not> using C<eval $VERSION>
+leads to even worse problems trying to specify a version number with
+C<use>:
+
+    # given $Foo::VERSION = "1.002_003"
+
+    use Foo 1.002_003; # fails!
+
+Underscore versions were a useful hack, but now it's time to move on and
+leave them behind.  But, if you really insist on underscores, the
+C<allow_decimal_underscore> option will let you.
+
+=head2 Using underscore in tuple $VERSION
+
+Yes, Perl allows this: C<v1.2.3_4>.  And even this: C<1.2.3_4>.  And
+this: C<v1.2_3>.  Or any of those in quotes.  (Maybe)
+
+But what happens is a random function of your version of Perl, your
+version of L<version.pm|version>, and your version of the CPAN toolchain.
+
+So you really shouldn't use underscores in version tuples, and this module
+won't let you.
+
 =head1 ATTRIBUTES
 
+=head2 allow_decimal_underscore
+
+Allows use of decimal versions with underscores.  Default is false.  (Version
+tuples with underscores are never allowed!)
+
 =head2 global
 
 If true, all occurrences of the version pattern will be replaced.  Otherwise,
