@@ -5,7 +5,7 @@ use warnings;
 package Dist::Zilla::Plugin::RewriteVersion;
 # ABSTRACT: Get and/or rewrite module versions to match distribution version
 
-our $VERSION = '0.009';
+our $VERSION = '0.012';
 
 use Moose;
 with(
@@ -15,9 +15,22 @@ with(
       { default_finders => [ ':InstallModules', ':ExecFiles' ], },
 );
 
+use Dist::Zilla::Plugin::BumpVersionAfterRelease::_Util;
 use namespace::autoclean;
 use version ();
 
+#pod =attr allow_decimal_underscore
+#pod
+#pod Allows use of decimal versions with underscores.  Default is false.  (Version
+#pod tuples with underscores are never allowed!)
+#pod
+#pod =cut
+
+has allow_decimal_underscore => (
+    is  => 'ro',
+    isa => 'Bool',
+);
+
 #pod =attr global
 #pod
 #pod If true, all occurrences of the version pattern will be replaced.  Otherwise,
@@ -30,9 +43,7 @@ has global => (
     isa => 'Bool',
 );
 
-my $assign_regex = qr{
-    our \s+ \$VERSION \s* = \s* (['"])($version::LAX)\1 \s* ;
-}x;
+my $assign_regex = assign_re();
 
 #pod =attr skip_version_provider
 #pod
@@ -45,6 +56,36 @@ my $assign_regex = qr{
 
 has skip_version_provider => ( is => ro =>, lazy => 1, default => undef );
 
+#pod =attr add_tarball_name
+#pod
+#pod If true, when the version is written, it will append a comment with the name of
+#pod the tarball it comes from.  This helps users track down the source of a
+#pod module if its name doesn't match the tarball name.  If the module is
+#pod a TRIAL release, that is also in the comment.  For example:
+#pod
+#pod     our $VERSION = '0.010'; # from Foo-Bar-0.010.tar.gz
+#pod     our $VERSION = '0.011'; # TRIAL from Foo-Bar-0.011-TRIAL.tar.gz
+#pod
+#pod This option defaults to false.
+#pod
+#pod =cut
+
+has add_tarball_name => ( is => ro =>, lazy => 1, default => undef );
+
+around dump_config => sub {
+    my ( $orig, $self ) = @_;
+    my $config = $self->$orig;
+
+    $config->{ +__PACKAGE__ } = {
+        finders => [ sort @{ $self->finder } ],
+        (
+            map { $_ => $self->$_ ? 1 : 0 } qw(global skip_version_provider add_tarball_name)
+        ),
+    };
+
+    return $config;
+};
+
 sub provide_version {
     my ($self) = @_;
     return if $self->skip_version_provider;
@@ -56,7 +97,8 @@ sub provide_version {
 
     my ( $quote, $version ) = $content =~ m{^$assign_regex[^\n]*$}ms;
 
-    $self->log_debug([ 'extracted version from main module: %s', $version ]) if $version;
+    $self->log_debug( [ 'extracted version from main module: %s', $version ] )
+      if $version;
     return $version;
 }
 
@@ -78,8 +120,12 @@ sub munge_file {
 
     my $version = $self->zilla->version;
 
-    $self->log_fatal("$version is not a valid version string")
-      unless version::is_lax($version);
+    $self->log_fatal(
+        "$version is not an allowed version string (maybe you need 'allow_decimal_underscore')"
+      )
+      unless $self->allow_decimal_underscore
+      ? is_loose_version($version)
+      : is_strict_version($version);
 
     if ( $self->rewrite_version( $file, $version ) ) {
         $self->log_debug( [ 'updating $VERSION assignment in %s', $file->name ] );
@@ -95,8 +141,16 @@ sub rewrite_version {
 
     my $content = $file->content;
 
-    my $comment = $self->zilla->is_trial ? ' # TRIAL' : '';
-    my $code = "our \$VERSION = '$version';$comment";
+    my $code = "our \$VERSION = '$version';";
+    $code .= " # TRIAL" if $self->zilla->is_trial;
+
+    if ( $self->add_tarball_name ) {
+        my $tarball = $self->zilla->archive_filename;
+        $code .= ( $self->zilla->is_trial ? "" : " #" ) . " from $tarball";
+    }
+
+    $code .= "\n\$VERSION = eval \$VERSION;"
+      if $version =~ /_/ and scalar( $version =~ /\./g ) <= 1;
 
     if (
         $self->global
@@ -130,7 +184,7 @@ Dist::Zilla::Plugin::RewriteVersion - Get and/or rewrite module versions to matc
 
 =head1 VERSION
 
-version 0.009
+version 0.012
 
 =head1 SYNOPSIS
 
@@ -151,6 +205,11 @@ the C<V> environment variable, similar to
 L<Git::NextVersion|Dist::Zilla::Plugin::Git::NextVersion>, in which case all
 the gathered files have their C<$VERSION> set to that value.
 
+By default, versions B<must> be "strict" -- decimal or 3+ part tuple with a
+leading "v".  The C<allow_decimal_underscore> option, if enabled, will also
+allow decimals to contain an underscore.  All other version forms are
+not allowed, including: "v1.2", "1.2.3" and "v1.2.3_4".
+
 Only the B<first> occurrence of a C<$VERSION> declaration in each file is
 relevant and/or affected (unless the L</global> attribute is set and it must
 exactly match this regular expression:
@@ -172,6 +231,11 @@ more details and usage examples.
 
 =head1 ATTRIBUTES
 
+=head2 allow_decimal_underscore
+
+Allows use of decimal versions with underscores.  Default is false.  (Version
+tuples with underscores are never allowed!)
+
 =head2 global
 
 If true, all occurrences of the version pattern will be replaced.  Otherwise,
@@ -184,6 +248,18 @@ instead of extracting it from the C<main_module>. Defaults to false.
 
 This enables hard-coding C<version => in C<dist.ini> among other tricks.
 
+=head2 add_tarball_name
+
+If true, when the version is written, it will append a comment with the name of
+the tarball it comes from.  This helps users track down the source of a
+module if its name doesn't match the tarball name.  If the module is
+a TRIAL release, that is also in the comment.  For example:
+
+    our $VERSION = '0.010'; # from Foo-Bar-0.010.tar.gz
+    our $VERSION = '0.011'; # TRIAL from Foo-Bar-0.011-TRIAL.tar.gz
+
+This option defaults to false.
+
 =for Pod::Coverage munge_files munge_file rewrite_version provide_version
 
 =head1 SEE ALSO
